@@ -1,4 +1,6 @@
 # cython: auto_cpdef=True
+import hashlib
+from io import StringIO
 import math
 from os import path
 from copy import deepcopy
@@ -11,6 +13,10 @@ from ._schema_common import (
     RESERVED_PROPERTIES,
     OPTIONAL_FIELD_PROPERTIES,
     RESERVED_FIELD_PROPERTIES,
+    JAVA_FINGERPRINT_MAPPING,
+    FINGERPRINT_ALGORITHMS,
+    RABIN_64,
+    rabin_fingerprint,
 )
 
 
@@ -655,3 +661,97 @@ def load_schema_ordered(ordered_schemas, *, _write_hint=True):
         _inject_schema(outer_schema, sub_schema)
 
     return outer_schema
+
+
+def to_parsing_canonical_form(schema):
+    """Returns a string represening the parsing canonical form of the schema.
+
+    For more details on the parsing canonical form, see here:
+    https://avro.apache.org/docs/current/spec.html#Parsing+Canonical+Form+for+Schemas
+    """
+    fo = StringIO()
+    _to_parsing_canonical_form(parse_schema(schema), fo)
+    return fo.getvalue()
+
+
+def _to_parsing_canonical_form(schema, fo):
+    # union schemas
+    if isinstance(schema, list):
+        fo.write("[")
+        for idx, s in enumerate(schema):
+            if idx != 0:
+                fo.write(",")
+            _to_parsing_canonical_form(s, fo)
+        fo.write("]")
+
+    # string schemas; this could be either a named schema or a primitive type
+    elif not isinstance(schema, dict):
+        fo.write(f'"{schema}"')
+
+    else:
+        # Remaining valid schemas must be dict types
+        schema_type = schema["type"]
+
+        if schema_type == "array":
+            fo.write(f'{{"type":"{schema_type}","items":')
+            _to_parsing_canonical_form(schema["items"], fo)
+            fo.write("}")
+
+        elif schema_type == "map":
+            fo.write(f'{{"type":"{schema_type}","values":')
+            _to_parsing_canonical_form(schema["values"], fo)
+            fo.write("}")
+
+        elif schema_type == "enum":
+            name = schema["name"]
+            fo.write(f'{{"name":"{name}","type":"{schema_type}","symbols":[')
+
+            for idx, symbol in enumerate(schema["symbols"]):
+                if idx != 0:
+                    fo.write(",")
+                fo.write(f'"{symbol}"')
+            fo.write("]}")
+
+        elif schema_type == "fixed":
+            name = schema["name"]
+            size = schema["size"]
+            fo.write(f'{{"name":"{name}","type":"{schema_type}","size":{size}}}')
+
+        elif schema_type == "record" or schema_type == "error":
+            name = schema["name"]
+            fo.write(f'{{"name":"{name}","type":"record","fields":[')
+
+            for idx, field in enumerate(schema["fields"]):
+                if idx != 0:
+                    fo.write(",")
+                name = field["name"]
+                fo.write(f'{{"name":"{name}","type":')
+                _to_parsing_canonical_form(field["type"], fo)
+                fo.write("}")
+            fo.write("]}")
+
+        elif schema_type in PRIMITIVES:
+            fo.write(f'"{schema_type}"')
+
+
+def fingerprint(parsing_canonical_form, algorithm):
+    """Returns a string represening a fingerprint/hash of the parsing canonical
+    form of a schema.
+
+    For more details on the fingerprint, see here:
+    https://avro.apache.org/docs/current/spec.html#schema_fingerprints
+    """
+    if algorithm not in FINGERPRINT_ALGORITHMS:
+        raise ValueError(
+            f"Unknown schema fingerprint algorithm {algorithm}. "
+            + f"Valid values include: {FINGERPRINT_ALGORITHMS}"
+        )
+
+    # Fix Java names
+    algorithm = JAVA_FINGERPRINT_MAPPING.get(algorithm, algorithm)
+
+    if algorithm == RABIN_64:
+        return rabin_fingerprint(parsing_canonical_form.encode())
+
+    h = hashlib.new(algorithm, parsing_canonical_form.encode())
+    return h.hexdigest()
